/*----------------------------------------------------------------------+
|                                                                       |
|   MessageTracer - MessageTracer source code. 							|
|                                                                       |
+----------------------------------------------------------------------*/

#include "StdAfx.h"
#include "MessageTracer.h"
#include <dlogids.h>
#include <msdialog.fdf>
#include <msrmgr.h>
#include <msvec.fdf>
#include <mdlerrs.h>
#include <NotificationManager.h>
///////////////////////////////////////////////////////////////////////
//	Implementation
//	static method to expand MicroStation's message center dialog
void	CMessageTracer::OpenMessageCenter		()
{
	mdlDialog_open (NULL, DIALOGID_MessageCenter);
}
///////////////////////////////////////////////////////////////////////
//	Output to MicroStation's message center.  If verbose_ is empty,
//	send terse_ to the message center list and detail window;
//	if verbose_ is not empty, send it to the detail window
void	CMessageTracer::Output					(OutputMessagePriority	priority,
												bool					popWarning)	const
{
	using namespace Bentley::Ustn;
	NotifyMessageDetails	message (priority, terse_.c_str (), verbose_.empty ()? terse_.c_str (): verbose_.c_str (), 0, popWarning? MESSAGE_ALERT_BALLOON: MESSAGE_ALERT_NONE);
	NotificationManager::OutputMessage (message);
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Append					(const wchar_t*			msg)
{
	terse_		+= msg;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Append					(const std::wstring&	msg)
{
	terse_		+= msg;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Format					(int					list_id,
												int						msg_id,
												const wchar_t*			s1)
{
	MSWChar		buffer	[128];
	MSWChar		format	[128];
	int			status	= mdlResource_loadFromStringListW   (format, NULL, list_id, msg_id);
	if (CheckStatus (status, list_id, msg_id))
	{
		swprintf (buffer, format, s1);
		terse_	= buffer;
	}
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Format					(int					list_id,
												int						msg_id,
												int						value)
{
	MSWChar		buffer	[128];
	MSWChar		format	[128];
	int			status	= mdlResource_loadFromStringListW   (format, NULL, list_id, msg_id);
	if (CheckStatus (status, list_id, msg_id))
	{
		swprintf (buffer, format, value);
		terse_	= buffer;
	}
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Format					(int					list_id,
												int						msg_id,
												const DPoint3d&			p,
												double					scale)
{
	MSWChar		buffer	[128];
	MSWChar		format	[128];
	int			status	= mdlResource_loadFromStringListW   (format, NULL, list_id, msg_id);
	if (CheckStatus (status, list_id, msg_id))
	{
		DPoint3d	out;
		mdlVec_scale (reinterpret_cast<DVec3d*>(&out), reinterpret_cast<DVec3d const*>(&p), scale);
		swprintf (buffer, format, out.x, out.y, out.z);
		terse_	= buffer;
	}
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Format					(const std::wstring&	format,
												const std::wstring&		s1)
{
	Format (format.c_str (), s1.c_str ());
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Format					(const MSWChar*			format,
												const MSWChar*			s1)
{
	MSWChar		buffer	[128];
	swprintf (buffer, format, s1);
	terse_		= buffer;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Format					(const wchar_t*			format,
												const char*				s1)
{
	MSWChar		buffer	[128];
	swprintf (buffer, format, s1);
	terse_		= buffer;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Format					(const wchar_t*			format,
												int						value)
{
	MSWChar		buffer	[128];
	swprintf (buffer, format, value);
	terse_		= buffer;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Format					(const wchar_t*			format,
												double					value)
{
	MSWChar		buffer	[128];
	swprintf (buffer, format, value);
	terse_		= buffer;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Format					(const wchar_t*			format,
												const DPoint3d&			p,
												double					scale)
{
	MSWChar		buffer	[128];
	DPoint3d	out;
	mdlVec_scale (reinterpret_cast<DVec3d*>(&out), reinterpret_cast<DVec3d const*>(&p), scale);
	swprintf (buffer, format, out.x, out.y, out.z);
	terse_		= buffer;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Message					(const wchar_t*			msg)
{
	terse_		= msg;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Message					(const std::wstring&	msg)
{
	terse_		= msg;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Verbose					(const wchar_t*			msg)
{
	verbose_	= msg;
}
///////////////////////////////////////////////////////////////////////
void	CMessageTracer::Verbose					(const std::wstring&	msg)
{
	verbose_	= msg;
}
///////////////////////////////////////////////////////////////////////
bool	CMessageTracer::CheckStatus				(int					status,
												int						list_id,
												int						msg_id)
{
	using namespace Bentley::Ustn;
	bool	rc	(false);
	switch (status)
	{
		case SUCCESS:
			rc	= true;
			break;
		case MDLERR_RSCFILENOTFOUND:
		{
			MSWChar	buffer	[128];
			swprintf (buffer, L"CMessageTracer: message list resource not found");
			NotifyMessageDetails	message (MESSAGE_WARNING, buffer, buffer, 0, MESSAGE_ALERT_NONE);
			NotificationManager::OutputMessage (message);
			terse_	= L"";
			break;
		}
		case MDLERR_RSCNOTFOUND:
		{
			MSWChar	buffer	[128];
			swprintf (buffer, L"CMessageTracer: invalid message list %d", list_id);
			NotifyMessageDetails	message (MESSAGE_WARNING, buffer, buffer, 0, MESSAGE_ALERT_NONE);
			NotificationManager::OutputMessage (message);
			terse_	= L"";
			break;
		}
	}
	return rc;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator=(const wchar_t*		msg)
{
	terse_	= msg;
	return *this;
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const wchar_t*			msg)
:	terse_								(msg? msg: L""),
	verbose_							(L"")
{
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const std::wstring&	msg)
:	terse_								(msg),
	verbose_							(L"")
{
}
///////////////////////////////////////////////////////////////////////
//	static helper
std::wstring	CMessageTracer::GetLastError	()
{
	LPVOID	lpMsgBuf	= NULL;
	TCHAR*	errorMessage;
	DWORD	error = ::GetLastError ();

	if (::FormatMessage (
					FORMAT_MESSAGE_ALLOCATE_BUFFER |
					FORMAT_MESSAGE_FROM_SYSTEM |
				   	FORMAT_MESSAGE_IGNORE_INSERTS,
				   	NULL,
					error,
					0, // Default language
					(LPTSTR) &lpMsgBuf,
					0,
				   	NULL))
	{
					errorMessage = static_cast<TCHAR*>(lpMsgBuf);
	}
#if defined (_UNICODE)
	std::wstring	msg	= errorMessage;
#else
	wchar_t			buffer	[128];
	mbstowcs (buffer, errorMessage, 128);
	std::wstring	msg	= buffer;
#endif
	if (lpMsgBuf)
		::LocalFree (lpMsgBuf);
	return msg;
}
///////////////////////////////////////////////////////////////////////
