#if !defined(MESSAGE_TRACER_H_INCLUDED_)
#define MESSAGE_TRACER_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
/*----------------------------------------------------------------------+
|                                                                       |
|   MessageTracer - MessageTracer source code. 							|
|                                                                       |
+----------------------------------------------------------------------*/
#include <msmodel.fdf>
#include <string>

class CMessageTracer
{
	std::wstring	terse_;
	std::wstring	verbose_;
	bool			CheckStatus	(int					status,
								int						list_id,
								int						msg_id);
public:
	///////////////////////////////////////////////////////////////////
	//	Implementation
	///////////////////////////////////////////////////////////////////
	//	Open MicroStation's message center dialog
	static	void			OpenMessageCenter	();
	//	Send a message to MicroStation's message center. By default,
	//	the message is shown with DEBUG priority (user must turn on
	//	debug messages in the message center.
	static std::wstring		GetLastError	();
	void					Output			(OutputMessagePriority	priority	= MESSAGE_DEBUG,
											bool					popAlert	= false)	const;
	///////////////////////////////////////////////////////////////////
	//	Provide an alternative message to the current message
	void					Message			(const wchar_t*			msg);
	void					Message			(const std::wstring&	msg);
	///////////////////////////////////////////////////////////////////
	//	Format an alternative message to the current message
	void					Format			(int					list_id,
											int						msg_id,
											const wchar_t*			s1		= NULL);
	void					Format			(int					list_id,
											int						msg_id,
											int						value);
	void					Format			(int					list_id,
											int						msg_id,
											const DPoint3d&			p,
											double					scale	= 1.0 / mdlModelRef_getUorPerMaster (ACTIVEMODEL));
	void					Format			(const wchar_t*			format,
											const wchar_t*			s1);
	void					Format			(const wchar_t*			format,
											const char*				s1);
	void					Format			(const wchar_t*			format,
											int						value);
	void					Format			(const wchar_t*			format,
											double					value);
	void					Format			(const wchar_t*			format,
											const DPoint3d&			value,
											double					scale	= 1.0 / mdlModelRef_getUorPerMaster (ACTIVEMODEL));
	void					Format			(const wchar_t*			format,
											const DVec3d&			value,
											double					scale	= 1.0 / mdlModelRef_getUorPerMaster (ACTIVEMODEL))
	{
		Format (format, static_cast<const DPoint3d&>(value), scale);
	}
	void					Format			(const std::wstring&	format,
											const std::wstring&	s1);
	///////////////////////////////////////////////////////////////////
	//	Provide a verbose message that differs from the terse message
	void					Verbose			(const wchar_t*			msg);
	void					Verbose			(const std::wstring&	msg);
	///////////////////////////////////////////////////////////////////
	//	Append new text to the current message
	void					Append			(const wchar_t*			suffix);
	void					Append			(const std::wstring&	suffix);
	///////////////////////////////////////////////////////////////////
	CMessageTracer&	operator=	(const wchar_t*			msg);
	///////////////////////////////////////////////////////////////////
	//	Construction
	CMessageTracer	(const wchar_t*			msg	= NULL);
	CMessageTracer	(const std::wstring&	msg);
};

#endif // !defined(MESSAGE_TRACER_H_INCLUDED_)
