/*----------------------------------------------------------------------+
|																		|
| Copyright , Bentley Systems, Incorporated, 							|
| All rights reserved.													|
|																		|
|																		|
| Limited permission is hereby granted to reproduce and modify this		|
| copyrighted material provided that the resulting code is used only in |
| conjunction with Bentley Systems products under the terms of the		|
| license agreement provided therein, and that this notice is retained	|
| in its entirety in any such reproduction or modification.				|
|																		|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   Additional material provided by LA Solutions Ltd.					|
|   www.la-solutions.co.uk                                              |
|                                                                       |
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   ViewSprites - ViewSprites source code. 								|
|                                                                       |
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   Include Files                                                       |
|                                                                       |
+----------------------------------------------------------------------*/

#if defined (CF_MAX)	//	Clipboard formats not used but invoked by Windows header macros
#undef CF_MAX
#endif

#include 	<windows.h>

#include "ViewDecorator.h"
#include "MessageTracer.h"

USING_NAMESPACE_BENTLEY
USING_NAMESPACE_BENTLEY_USTN
USING_NAMESPACE_BENTLEY_USTN_ELEMENT
USING_NAMESPACE_TEXT

/*---------------------------------------------------------------------------------**//**
* @description  ~DisplaySpriteSample
*	DisplaySpriteSample destructor
* @param 	none
+---------------+---------------+---------------+---------------+---------------+------*/
DisplaySpriteSample::~DisplaySpriteSample	()
{
	IViewManager::GetManager ().DropViewTransientHandler (this);
}
/*---------------------------------------------------------------------------------**//**
* @description  IconExists
*	Determines whether an icon resource exists in our DLL
* @param 	int				icon ID
* @param 	const char*		dllName
+---------------+---------------+---------------+---------------+---------------+------*/
bool	DisplaySpriteSample::IconExists			(int			iconId,
												const wchar_t*	dllName)
{
	//HICON		icon	= ::LoadIcon (iconId);
    HINSTANCE	dll		= GetModuleHandleW (dllName);
    if (NULL == dll)
    {
		CMessageTracer	msg;
		msg.Format (L"DisplaySpriteSample::IconExists: invalid module name '%s'", dllName);
		msg.Output (MESSAGE_WARNING);
		return false;
	}
	//	Do not use LR_SHARED for images that have non-standard sizes, that may change after loading, or that are loaded from a file
	//	The following line works but uses a hard-coded file name.  Presumably it's opening the icon file
	//	rather than obtaining the icon from the DLL resource data
	//HICON 		icon	= (HICON)::LoadImage (NULL, "ICON2.ico", IMAGE_ICON, 0, 0, LR_LOADFROMFILE);
	HICON 		icon	= (HICON)::LoadImage (dll, MAKEINTRESOURCE(iconId), IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR);
	const bool	Exists	(NULL != icon);
	if (Exists)
	{
		iconId_	= iconId;
		::DestroyIcon (icon);
	}
	else
	{
		std::wstring		problem	(CMessageTracer::GetLastError ());

		CMessageTracer	msg;
		msg.Format (L"DisplaySpriteSample::IconExists: unable to load icon ID %d", iconId);
		msg.Verbose (problem);
		msg.Output (MESSAGE_WARNING);
	}
	return Exists;
}
/*---------------------------------------------------------------------------------**//**
* @description  LoadSprite
* @param 	int				icon ID
* @param 	const char*		dllName
+---------------+---------------+---------------+---------------+---------------+------*/
bool	DisplaySpriteSample::LoadSprite			(int			iconId,
												const char*		dllName)
{
	wchar_t		dll	[128];
	mdlCnv_convertMultibyteToUnicode (dllName, -1, dll, 128);
	return LoadSprite (iconId, dll);
}
/*---------------------------------------------------------------------------------**//**
* @description  LoadSprite
* [Brien Bastings]
* We don't call LoadImage until GetIcon is called ... that's when we know if the sprite is valid.
* CreateFromIconResource always returns a new object.
* @param 	int				icon ID
* @param 	const char*		dllName

+---------------+---------------+---------------+---------------+---------------+------*/
bool	DisplaySpriteSample::LoadSprite			(int			iconId,
												const wchar_t*	dllName)
{
	bool		rc		(false);
    HINSTANCE	dll	= GetModuleHandleW (dllName);
    //	CreateFromIconResource always returns a new object [Brien Bastings]
	pSprite_	= ISprite::CreateFromIconResource (iconId, dll);
	return (NULL != pSprite_->GetIcon ());
}
/*---------------------------------------------------------------------------------**//**
* @description  _DrawTransients callback
* @param 	IViewContextP context
* @param 	bool isPreUpdate
+---------------+---------------+---------------+---------------+---------------+------*/
void	DisplaySpriteSample::_DrawTransients (IViewContextP context, bool isPreUpdate)
{
	//if (isPreUpdate || (NULL != context->GetIPickGeom ()))
	if (isPreUpdate)
		return;

	CMessageTracer	msg;
	msg	= L"DisplaySpriteSample::_DrawTransients";
	msg.Output ();

	DrawSpriteSample (context);
}
/*---------------------------------------------------------------------------------**//**
* @description  DrawSpriteSample
* @param 	IViewContextP	context
+---------------+---------------+---------------+---------------+---------------+------*/
void	DisplaySpriteSample::DrawSpriteSample	(IViewContextP	context)
{
	IViewDrawP	pViewDraw = context->GetIViewDraw ();
	if (pViewDraw)
	{
		CMessageTracer	msg;
		msg.Format (L"DrawSpriteSample: pViewDraw %s", NULL == pViewDraw? L"NULL": L"valid");
		msg.Output ();
		DVec3d		dirVec;
		//	This will follow the last data point on the view (not the current cursor position).
		//	statedata is an MDL global variable
		dirVec.x = statedata.inPoint.dpUors.x;
		dirVec.y = statedata.inPoint.dpUors.y;
		dirVec.z = statedata.inPoint.dpUors.z;
		//	This will follow the current cursor position.
		//	statedata is an MDL global variable
		dirVec.x = statedata.current.dpUors.x;
		dirVec.y = statedata.current.dpUors.y;
		dirVec.z = statedata.current.dpUors.z;
		pViewDraw->DrawSprite (pSprite_, &dirVec, NULL, 0);
	}
}



