/*----------------------------------------------------------------------+
|																		|
| Copyright , Bentley Systems, Incorporated, 							|
| All rights reserved.													|
|																		|
|																		|
| Limited permission is hereby granted to reproduce and modify this		|
| copyrighted material provided that the resulting code is used only in |
| conjunction with Bentley Systems products under the terms of the		|
| license agreement provided therein, and that this notice is retained	|
| in its entirety in any such reproduction or modification.				|
|																		|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   Additional material provided by LA Solutions Ltd.					|
|   www.la-solutions.co.uk                                              |
|                                                                       |
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   ViewSprites - ViewSprites source code. 								|
|                                                                       |
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   Include Files                                                       |
|                                                                       |
+----------------------------------------------------------------------*/
#include	<MicroStationAPI.h>
#include 	<windows.h>

#if defined (CF_MAX)	//	Clipboard formats not used but invoked by Windows header macros
#undef CF_MAX
#endif

#include    <mdl.h>
#include 	<cmdlist.h>

#include "MessageTracer.h"
#include "ViewSpritesCmd.h"
#include "ViewDecorator.h"		//	Our ViewDecorator class
#include "ViewSprites.h"

extern "C"
{
	//	Alias for ustnTaskId if compiling native code C or C++
	//	Link with mdlbltin.lib
	extern char*		ustnTaskIdP;
}	//	extern C

USING_NAMESPACE_BENTLEY
USING_NAMESPACE_BENTLEY_USTN

///////////////////////////////////////////////////////////////////////
//	The one and only instance of our ViewDecorator class
static DisplaySpriteSample s_SpriteSample;

///////////////////////////////////////////////////////////////////////
//	CMD_VIEWSPRITES_EXIT
extern "C" DLLEXPORT void ViewDeco_exit
(
char   *unparsed
)
{
	mdlInput_sendCommand (CMD_MDL_UNLOAD, mdlSystem_getCurrTaskID (),
		INPUTQ_EOQ, ustnTaskIdP, 0);
}
///////////////////////////////////////////////////////////////////////
//	CMD_VIEWSPRITES_SPRITE_ADD
extern "C" DLLEXPORT void ViewDeco_spriteAdd
(
char   *unparsed
)
{
	IViewManager::GetManager ().AddViewTransientHandler (&s_SpriteSample);
	CMessageTracer	msg (L"Add DisplaySpriteSample");
	msg.Output ();
}
///////////////////////////////////////////////////////////////////////
//	CMD_VIEWSPRITES_SPRITE_DELETE
extern "C" DLLEXPORT void ViewDeco_spriteDelete
(
char   *unparsed
)
{
	IViewManager::GetManager().DropViewTransientHandler (&s_SpriteSample);
	CMessageTracer	msg (L"Drop DisplaySpriteSample");
	msg.Output ();
}
//////////////////////////////////////////////////////////////////////
void	SetVersionNumber		()
{
	VersionNumber	v	= { 10, 4, 6, 0 };
	mdlSystem_setMdlAppVersionNumber (mdlSystem_getCurrMdlDesc (), &v);
}
/*---------------------------------------------------------------------------------**//**
* @description  MdlMain
* @param 	argc      The number of command line parameters sent to the application.
* @param 	argv[]    The array of strings sent to the application on the command line.
* @bsimethod 							BSI             06/03
+---------------+---------------+---------------+---------------+---------------+------*/
extern "C" DLLEXPORT  int MdlMain
(
int         argc,
char        *argv[]
)
{
	RscFileHandle  		rfHandle	= 0;

    mdlResource_openFile (&rfHandle, NULL, RSC_READONLY);

	CMessageTracer::OpenMessageCenter ();

    //	Load up the sprite when the app is loaded
    const std::wstring	hostDll		(L"ViewSprites.dll");
    const int			iconId		= IDI_ICON2;
    if (!s_SpriteSample.LoadSprite (iconId, hostDll))
    {
		CMessageTracer	inform;
		inform.Format (MESSAGELISTID_Messages, MESSAGEID_IconLoadError, iconId);
		inform.Output (MESSAGE_WARNING);
		ViewDeco_exit (NULL);
	}
    // Map command key-in to function
    static MdlCommandNumber cmdNumbers[] =
    {
         {ViewDeco_spriteAdd,			CMD_VIEWSPRITES_SPRITE_ADD		},
         {ViewDeco_spriteDelete,		CMD_VIEWSPRITES_SPRITE_DELETE	},
         {ViewDeco_exit,				CMD_VIEWSPRITES_EXIT			},
        0,
    };

    mdlSystem_registerCommandNumbers (cmdNumbers);

    mdlParse_loadCommandTable (NULL);

	SetVersionNumber ();

 	return SUCCESS;
}
